/*
 * mp32m3u - simple program to read the id3v1 tags from
 * a list of passed in mp3 files and output them
 * as http links to the mp3 with a pretty name
 *
 * Author: Robin Rawson-Tetley
 * Licence: Public Domain
 */

#define BASE_URL "http://slug/music"

#include <stdio.h>

char tag[129];
char* marker;
char* title;
char* artist;
char* album;
char* year;

int main(int argc, char* argv[]) {
    
    int i = 0;
    // Loop through command line arguments
    for (i = 1; i < argc; i++) {
    
        // Open each file
        FILE* f = fopen(argv[i], "r");

        // Parse the tag
        if (parse(f)) {

            // Dump the output
            printf("#EXTINF:-1,%s - %s (%s)\n", title, artist, album);
            printf("%s/%s\n", BASE_URL, argv[i]);

        }

        // Close the file
        fclose(f);
    }

}

// Strips whitespace from the end and makes sure
// we have a terminating char
void strip(char* c, int clen) {
	int i = 0;
	for (i = clen-1; i > 0; i--) {
		if (*(c + i) != '\0' && *(c + i) != ' ' && *(c + i) != '\n' && *(c + i) != '\r' && *(c + i) != '\t') {
			// first non-whitespace char
			*(c + i + 1) = '\0';
			return;
		}
	}
	// Nastiness, just blank the string
	*c = '\0';
}

// Parses the mp3 tag from a file, returns 0 if 
// there is no tag
int parse(FILE* f) {

    // Seek to the last 128 bytes of the file
    fseek(f, -128, SEEK_END);
    fread(tag, 1, 128, f);

    // Position pointers
    marker = tag;
    title = marker + 3;
    strip(title, 30);
    artist = title + 30;
    strip(artist, 30);
    album = artist + 30;
    strip(album, 30);
    year = album + 30;
    strip(year, 4);

    if (strncmp(marker, "TAG", 3) == 0)
        return 1;
    else
        return 0;
}


